@@ -118,7 +118,7 @@ struct _GarconMenuDirectoryPrivate
 
 
 /* TODO, maybe implement the GarconMenuElement interface */
-G_DEFINE_TYPE (GarconMenuDirectory, garcon_menu_directory, G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (GarconMenuDirectory, garcon_menu_directory, G_TYPE_OBJECT)
 
 
 
@@ -127,8 +127,6 @@ garcon_menu_directory_class_init (GarconMenuDirectoryClass *klass)
 {
   GObjectClass *gobject_class;
 
-  g_type_class_add_private (klass, sizeof(GarconMenuDirectoryPrivate));
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = garcon_menu_directory_finalize;
   gobject_class->get_property = garcon_menu_directory_get_property;
@@ -212,7 +210,7 @@ garcon_menu_directory_class_init (GarconMenuDirectoryClass *klass)
 static void
 garcon_menu_directory_init (GarconMenuDirectory *directory)
 {
-  directory->priv = G_TYPE_INSTANCE_GET_PRIVATE (directory, GARCON_TYPE_MENU_DIRECTORY, GarconMenuDirectoryPrivate);
+  directory->priv = garcon_menu_directory_get_instance_private (directory);
   directory->priv->file = NULL;
   directory->priv->name = NULL;
   directory->priv->icon_name = NULL;
@@ -328,7 +326,7 @@ garcon_menu_directory_set_property (GObject      *object,
 
 
 /**
- * garcon_menu_directory_new:
+ * garcon_menu_directory_new: (method)
  * @file : a #GFile
  *
  * Create a new #GarconMenuDirectory for @file. You most likely never
@@ -394,12 +392,12 @@ garcon_menu_directory_new (GFile *file)
 
 /**
  * garcon_menu_directory_get_file:
- * @directory : a #GarconMenuDirectory
+ * @directory: a #GarconMenuDirectory
  *
- * Get the file for @directory.
+ * Get the #GFile for @directory. The returned object should be
+ * unreffed with g_object_unref() when no longer needed.
  *
- * Returns: a #GFile. The returned object
- * should be unreffed with g_object_unref() when no longer needed.
+ * Returns: (transfer full): a #GFile
  */
 GFile *
 garcon_menu_directory_get_file (GarconMenuDirectory *directory)
@@ -522,7 +520,7 @@ garcon_menu_directory_get_icon_name (GarconMenuDirectory *directory)
 /**
  * garcon_menu_directory_set_icon_name:
  * @directory : a #GarconMenuDirectory
- * @icon_name : the new icon name for @directory.
+ * @icon_name      : the new icon name for @directory.
  *
  * Set the icon name of @directory.
  */
