@@ -36,6 +36,12 @@
 
 
 
+#ifndef G_KEY_FILE_DESKTOP_KEY_KEYWORDS
+#define G_KEY_FILE_DESKTOP_KEY_KEYWORDS "Keywords"
+#endif
+
+
+
 /* Property identifiers */
 enum
 {
@@ -722,13 +728,31 @@ garcon_menu_item_lists_equal (GList *list1,
 static gchar *
 garcon_menu_item_url_exec (XfceRc *rc)
 {
+  GString     *string;
   const gchar *url;
   gchar       *url_exec = NULL;
 
   /* Support Type=Link items */
   url = xfce_rc_read_entry_untranslated (rc, G_KEY_FILE_DESKTOP_KEY_URL, NULL);
   if (url != NULL)
-    url_exec = g_strdup_printf ("exo-open '%s'", url);
+    {
+#if GLIB_CHECK_VERSION (2, 68, 0)
+      string = g_string_new (url);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+      g_string_replace (string, "%", "%%", 0);
+G_GNUC_END_IGNORE_DEPRECATIONS
+#else
+      string = g_string_sized_new (2 * strlen (url));
+      for (const gchar *p = url; *p != '\0'; p++)
+        if (*p == '%')
+          string = g_string_append (string, "%%");
+        else
+          string = g_string_append_c (string, *p);
+      string = g_string_append_c (string, '\0');
+#endif
+      url_exec = g_strdup_printf ("exo-open '%s'", string->str);
+      g_string_free (string, TRUE);
+    }
 
   return url_exec;
 }
@@ -1357,7 +1381,7 @@ garcon_menu_item_set_desktop_id (GarconMenuItem *item,
  *
  * Returns list of categories
  *
- * Returns: (element-type utf8) (transfer full):
+ * Returns: (element-type utf8) (transfer none):
  */
 GList*
 garcon_menu_item_get_categories (GarconMenuItem *item)
@@ -1394,7 +1418,7 @@ garcon_menu_item_set_categories (GarconMenuItem *item,
  * garcon_menu_item_get_keywords:
  * @item: a #GarconMenuItem
  *
- * Returns: (element-type utf8) (transfer full):
+ * Returns: (element-type utf8) (transfer none):
  */
 GList*
 garcon_menu_item_get_keywords (GarconMenuItem *item)
@@ -1814,7 +1838,7 @@ garcon_menu_item_has_keyword (GarconMenuItem *item,
  * garcon_menu_item_get_actions:
  * @item: a #GarconMenuItem
  *
- * Returns: (element-type GarconMenuItemAction) (transfer full):
+ * Returns: (element-type GarconMenuItemAction) (transfer container):
  */
 GList *
 garcon_menu_item_get_actions (GarconMenuItem *item)
@@ -1841,7 +1865,7 @@ garcon_menu_item_get_actions (GarconMenuItem *item)
  * @item: a #GarconMenuItem
  * @action_name:
  *
- * Returns: (nullable) (transfer full): a #GarconMenuItemAction
+ * Returns: (nullable) (transfer none): a #GarconMenuItemAction
  */
 GarconMenuItemAction *
 garcon_menu_item_get_action (GarconMenuItem *item,
