@@ -21,7 +21,7 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#if !defined(GARCON_INSIDE_GARCON_H) && !defined(GARCON_COMPILATION)
+#if !defined(_GARCON_INSIDE_GARCON_H) && !defined(GARCON_COMPILATION)
 #error "Only <garcon/garcon.h> can be included directly. This file may disappear or change contents."
 #endif
 
@@ -45,8 +45,6 @@ typedef struct _GarconMenuItem        GarconMenuItem;
 #define GARCON_IS_MENU_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GARCON_TYPE_MENU_ITEM))
 #define GARCON_MENU_ITEM_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GARCON_TYPE_MENU_ITEM, GarconMenuItemClass))
 
-#define  G_KEY_FILE_DESKTOP_KEY_KEYWORDS "Keywords"
-
 struct _GarconMenuItemClass
 {
   GObjectClass __parent__;
