@@ -23,7 +23,7 @@
 
 #include <gio/gio.h>
 
-#define GARCON_INSIDE_GARCON_H
+#define _GARCON_INSIDE_GARCON_H
 
 #include <garcon/garcon-config.h>
 #include <garcon/garcon-menu-directory.h>
@@ -31,6 +31,7 @@
 #include <garcon/garcon-environment.h>
 #include <garcon/garcon-menu.h>
 #include <garcon/garcon-menu-item.h>
+#include <garcon/garcon-menu-item-action.h>
 #include <garcon/garcon-menu-item-cache.h>
 #include <garcon/garcon-menu-item-pool.h>
 #include <garcon/garcon-menu-node.h>
@@ -39,6 +40,6 @@
 #include <garcon/garcon-menu-separator.h>
 #include <garcon/garcon-menu-tree-provider.h>
 
-#undef GARCON_INSIDE_GARCON_H
+#undef _GARCON_INSIDE_GARCON_H
 
 #endif /* !__GARCON_H__ */
